==> billiards-0.1.tar.gz: billiards-0.1/src/universal.c <==
/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ode/ode.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include "universal.h"

@implementation Universal

-(Joint *) init
{
    [super init];
    
    self->joint = dJointCreateUniversal ([self world], NULL);

    self->anchor[0] = 0;
    self->anchor[1] = 0;
    self->anchor[2] = 0;

    self->axes[0][0] = 1;
    self->axes[0][1] = 0;
    self->axes[0][2] = 0;

    self->axes[1][0] = 0;
    self->axes[1][1] = 1;
    self->axes[1][2] = 0;
    
    self->stops[0][0] = -dInfinity;
    self->stops[0][1] = dInfinity;

    self->stops[1][0] = -dInfinity;
    self->stops[1][1] = dInfinity;

    return self;
}

-(void) get: (lua_State *)L
{
    const char *k;
    int i, j;
    
    k = lua_tostring(L, -1);

    if (!strcmp(k, "anchor")) {
	dJointGetUniversalAnchor ([self joint], self->anchor);

        lua_newtable(L);
        
        for(j = 0; j < 3; j += 1) {
            lua_pushnumber(L, self->anchor[j]);
            lua_rawseti(L, -2, j + 1);
        }
    } else if (!strcmp(k, "axes")) {
	dJointGetUniversalAxis1 ([self joint], self->axes[0]);
	dJointGetUniversalAxis2 ([self joint], self->axes[1]);

	lua_newtable(L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_newtable(L);
        
	    for(i = 0; i < 3; i += 1) {
		lua_pushnumber(L, self->axes[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "motor")) {
	lua_newtable(L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_newtable(L);
        
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber(L, self->motor[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "stops")) {
	lua_newtable(L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_newtable(L);

	    lua_newtable(L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber(L, self->stops[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }
	    lua_rawseti(L, -2, 1);
	
	    lua_newtable(L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber(L, self->hardness[j][i]);
		lua_rawseti(L, -2, i + 1);
	    }
	    lua_rawseti(L, -2, 2);

	    lua_pushnumber(L, bounce[j]);
	    lua_rawseti(L, -2, 3);

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "tolerance")) {
	lua_newtable(L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_pushnumber(L, self->tolerance[j]);

	    lua_rawseti(L, -2, j + 1);
	}
    } else if (!strcmp(k, "state")) {
	dReal state[4];

	if ([self joint]) {
	    state[0] = dJointGetUniversalAngle1 ([self joint]);
	    state[1] = dJointGetUniversalAngle2 ([self joint]);
	    state[2] = dJointGetUniversalAngle1Rate ([self joint]);
	    state[3] = dJointGetUniversalAngle2Rate ([self joint]);

	    lua_newtable(L);
        
	    for(j = 0 ; j < 4 ; j += 1) {
		lua_pushnumber(L, state[j]);
		
		lua_rawseti(L, -2, j + 1);
	    }
	} else {
	    lua_pushnil(L);
	}
    } else {
	[super get: L];
    }
}

-(void) set: (lua_State *)L
{
    const char *k;
    int i, j;
    
    k = lua_tostring(L, -2);

    if (!strcmp(k, "anchor")) {
        if(lua_istable(L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti(L, 3, j + 1);
                self->anchor[j] = lua_tonumber(L, -1);
                
                lua_pop(L, 1);
            }

	    dJointSetUniversalAnchor ([self joint],
				      self->anchor[0],
				      self->anchor[1],
				      self->anchor[2]);
        }
    } else if (!strcmp(k, "axes")) {
        if(lua_istable(L, 3)) {
	    for(j = 0 ; j < 2 ; j += 1) {
		lua_rawgeti(L, 3, j + 1);
		
		if(lua_istable(L, -1)) {
		    for(i = 0 ; i < 3 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);
			self->axes[j][i] = lua_tonumber(L, -1);
                
			lua_pop(L, 1);
		    }
		}

		lua_pop (L, 1);
	    }

	    dJointSetUniversalAxis1 ([self joint],
				     self->axes[0][0],
				     self->axes[0][1],
				     self->axes[0][2]);

	    dJointSetUniversalAxis2 ([self joint],
				     self->axes[1][0],
				     self->axes[1][1],
				     self->axes[1][2]);
	}
    } else if (!strcmp(k, "motor")) {
        if(lua_istable(L, 3)) {
	    for(j = 0 ; j < 2 ; j += 1) {
		lua_rawgeti(L, 3, j + 1);
		
		if(lua_istable(L, -1)) {
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);
			self->motor[j][i] = lua_tonumber(L, -1);
                
			lua_pop(L, 1);
		    }

		    dJointSetUniversalParam ([self joint],
					     dParamVel + dParamGroup * j,
					     self->motor[j][0]);
		    
		    dJointSetUniversalParam ([self joint],
					     dParamFMax + dParamGroup * j,
					     self->motor[j][1]);
		}

		lua_pop (L, 1);
	    }
	}
    } else if (!strcmp(k, "stops")) {
        if(lua_istable(L, 3)) {
	    for(j = 0 ; j < 2 ; j += 1) {
		lua_rawgeti(L, 3, j + 1);
		
		if(lua_istable(L, -1)) {
		    lua_rawgeti(L, -1, 1);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);
		
			self->stops[j][i] = lua_tonumber(L, -1);
		
			lua_pop(L, 1);
		    }
		    lua_pop(L, 1);

		    lua_rawgeti(L, -1, 2);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti(L, -1, i + 1);

			self->hardness[j][i] = lua_tonumber(L, -1);
			
			lua_pop(L, 1);
		    }
		    lua_pop(L, 1);
	
		    lua_rawgeti(L, -1, 3);
		    bounce[j] = lua_tonumber(L, -1);
		    lua_pop(L, 1);


		    /* Resetting the self->stops makes sure that lo remains
		       smaller than hi between calls. */

		    dJointSetUniversalParam ([self joint],
					     dParamLoStop + dParamGroup * j,
					     -dInfinity);
		    
		    dJointSetUniversalParam ([self joint],
					     dParamHiStop + dParamGroup * j,
					     dInfinity);
		    
		    dJointSetUniversalParam ([self joint],
					     dParamLoStop + dParamGroup * j,
					     self->stops[j][0]);

		    dJointSetUniversalParam ([self joint],
					     dParamHiStop + dParamGroup * j,
					     self->stops[j][1]);

		    dJointSetUniversalParam ([self joint],
					     dParamStopCFM + dParamGroup * j,
					     self->hardness[j][0]);

		    dJointSetUniversalParam ([self joint],
					     dParamStopERP + dParamGroup * j,
					     self->hardness[j][1]);

		    dJointSetUniversalParam ([self joint],
					     dParamBounce + dParamGroup * j,
					     bounce[j]);
		}

		lua_pop (L, 1);
	    }
	}
    } else if (!strcmp(k, "tolerance")) {
        if(lua_istable(L, 3)) {
            for(j = 0 ; j < 2 ; j += 1) {
                lua_rawgeti(L, 3, j + 1);
                self->tolerance[j] = lua_tonumber(L, -1);
                
                lua_pop(L, 1);
            }

	    dJointSetUniversalParam ([self joint],
				     dParamCFM + dParamGroup * j,
				     self->tolerance[j]);
        }
    } else if (!strcmp(k, "bodies")) {
	[super set: L];
	
	dJointSetUniversalAnchor ([self joint],
				  self->anchor[0],
				  self->anchor[1],
				  self->anchor[2]);

	dJointSetUniversalAxis1 ([self joint],
				 self->axes[0][0],
				 self->axes[0][1],
				 self->axes[0][2]);

	dJointSetUniversalAxis2 ([self joint],
				 self->axes[1][0],
				 self->axes[1][1],
				 self->axes[1][2]);
    }
}

@end

==> billiards-0.2.tar.gz: billiards-0.2/src/universal.c <==
/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ode/ode.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include "universal.h"

@implementation Universal

-(Joint *) init
{
    [super init];
    
    self->joint = dJointCreateUniversal (_WORLD, NULL);

    self->anchor[0] = 0;
    self->anchor[1] = 0;
    self->anchor[2] = 0;

    self->axes[0][0] = 1;
    self->axes[0][1] = 0;
    self->axes[0][2] = 0;

    self->axes[1][0] = 0;
    self->axes[1][1] = 1;
    self->axes[1][2] = 0;
    
    self->stops[0][0] = -dInfinity;
    self->stops[0][1] = dInfinity;

    self->stops[1][0] = -dInfinity;
    self->stops[1][1] = dInfinity;

    return self;
}

-(void) get
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -1);

    if (!xstrcmp(k, "anchor")) {
	dJointGetUniversalAnchor ([self joint], self->anchor);

        lua_newtable (_L);
        
        for(j = 0; j < 3; j += 1) {
            lua_pushnumber (_L, self->anchor[j]);
            lua_rawseti (_L, -2, j + 1);
        }
    } else if (!xstrcmp(k, "axes")) {
	dJointGetUniversalAxis1 ([self joint], self->axes[0]);
	dJointGetUniversalAxis2 ([self joint], self->axes[1]);

	lua_newtable (_L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 3; i += 1) {
		lua_pushnumber (_L, self->axes[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "motor")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->motor[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "stops")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_newtable (_L);

	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->stops[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 1);
	
	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->hardness[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 2);

	    lua_pushnumber (_L, bounce[j]);
	    lua_rawseti (_L, -2, 3);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "tolerance")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 2 ; j += 1) {
	    lua_pushnumber (_L, self->tolerance[j]);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "state")) {
	dReal state[4];

	if ([self joint]) {
	    state[0] = dJointGetUniversalAngle1 ([self joint]);
	    state[1] = dJointGetUniversalAngle2 ([self joint]);
	    state[2] = dJointGetUniversalAngle1Rate ([self joint]);
	    state[3] = dJointGetUniversalAngle2Rate ([self joint]);

	    lua_newtable (_L);
        
	    for(j = 0 ; j < 4 ; j += 1) {
		lua_pushnumber (_L, state[j]);
		
		lua_rawseti (_L, -2, j + 1);
	    }
	} else {
	    lua_pushnil (_L);
	}
    } else {
	[super get];
    }
}

-(void) set
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -2);

    if (!xstrcmp(k, "anchor")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->anchor[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);
            }

	    dJointSetUniversalAnchor ([self joint],
				      self->anchor[0],
				      self->anchor[1],
				      self->anchor[2]);
        }
    } else if (!xstrcmp(k, "axes")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 2 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 3 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->axes[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }
		}

		lua_pop (_L, 1);
	    }

	    dJointSetUniversalAxis1 ([self joint],
				     self->axes[0][0],
				     self->axes[0][1],
				     self->axes[0][2]);

	    dJointSetUniversalAxis2 ([self joint],
				     self->axes[1][0],
				     self->axes[1][1],
				     self->axes[1][2]);
	}
    } else if (!xstrcmp(k, "motor")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 2 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->motor[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }

		    dJointSetUniversalParam ([self joint],
					     dParamVel + dParamGroup * j,
					     self->motor[j][0]);
		    
		    dJointSetUniversalParam ([self joint],
					     dParamFMax + dParamGroup * j,
					     self->motor[j][1]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "stops")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 2 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    lua_rawgeti (_L, -1, 1);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
		
			self->stops[j][i] = lua_tonumber (_L, -1);
		
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);

		    lua_rawgeti (_L, -1, 2);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);

			self->hardness[j][i] = lua_tonumber (_L, -1);
			
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);
	
		    lua_rawgeti (_L, -1, 3);
		    bounce[j] = lua_tonumber (_L, -1);
		    lua_pop (_L, 1);


		    /* Resetting the self->stops makes sure that lo remains
		       smaller than hi between calls. */

		    dJointSetUniversalParam ([self joint],
					     dParamLoStop + dParamGroup * j,
					     -dInfinity);
		    
		    dJointSetUniversalParam ([self joint],
					     dParamHiStop + dParamGroup * j,
					     dInfinity);
		    
		    dJointSetUniversalParam ([self joint],
					     dParamLoStop + dParamGroup * j,
					     self->stops[j][0]);

		    dJointSetUniversalParam ([self joint],
					     dParamHiStop + dParamGroup * j,
					     self->stops[j][1]);

		    dJointSetUniversalParam ([self joint],
					     dParamStopCFM + dParamGroup * j,
					     self->hardness[j][0]);

		    dJointSetUniversalParam ([self joint],
					     dParamStopERP + dParamGroup * j,
					     self->hardness[j][1]);

		    dJointSetUniversalParam ([self joint],
					     dParamBounce + dParamGroup * j,
					     bounce[j]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "tolerance")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 2 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->tolerance[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);
            }

	    dJointSetUniversalParam ([self joint],
				     dParamCFM + dParamGroup * j,
				     self->tolerance[j]);
        }
    } else if (!xstrcmp(k, "bodies")) {
	[super set];
	
	dJointSetUniversalAnchor ([self joint],
				  self->anchor[0],
				  self->anchor[1],
				  self->anchor[2]);

	dJointSetUniversalAxis1 ([self joint],
				 self->axes[0][0],
				 self->axes[0][1],
				 self->axes[0][2]);

	dJointSetUniversalAxis2 ([self joint],
				 self->axes[1][0],
				 self->axes[1][1],
				 self->axes[1][2]);
    } else {
	[super set];
    }
}

@end
